@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -79,6 +79,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
   /* See if this operator needs any parameters. If so, pop them. */
   switch(op)
     {
+    case GAL_ARITHMETIC_OP_STITCH:
     case GAL_ARITHMETIC_OP_QUANTILE:
       numparams=1;
       break;
@@ -1019,6 +1020,10 @@ arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
                            ? OPERATOR_PREFIX_LENGTH_TOFILEFREE
                            : OPERATOR_PREFIX_LENGTH_TOFILE     ];
 
+  /* Make sure that if the file exists, it is deleted. */
+  gal_checkset_writable_remove(filename, p->cp.keep,
+                               p->cp.dontdelete);
+
   /* Save it to a file. */
   popped->wcs=p->refdata.wcs;
   if(popped->ndim==1 && p->onedasimage==0)
@@ -1031,10 +1036,8 @@ arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
 
   /* Reset the WCS to NULL and put it back on the stack. */
   popped->wcs=NULL;
-  if(freeflag)
-    gal_data_free(popped);
-  else
-    operands_add(p, NULL, popped);
+  if(freeflag) gal_data_free(popped);
+  else         operands_add(p, NULL, popped);
 }
 
 
@@ -1042,30 +1045,20 @@ arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
 
 
 void
-arithmetic_unique(struct arithmeticparams *p, char *token, int operator)
+arithmetic_add_dimension(struct arithmeticparams *p, char *token,
+                         int operator)
 {
-  /* Pass the popped operand to the statistics library. */
-  gal_data_t *input = gal_statistics_unique(operands_pop(p, token), 1);
-  operands_add(p, NULL, input);
-}
-
-
-
-
-
-void
-arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
-{
-  gal_data_t *out=NULL;
+  size_t one=1;
+  gal_data_t *ref=NULL, *out=NULL;
+  size_t i, j, num, dsize[3], nbytes=0;
   gal_data_t *tmp = operands_pop(p, token);
-  size_t i, num, dsize[3], ndim=3, nbytes=0;
 
   /* Make sure the first operand is a number. */
   if(tmp->size!=1)
     error(EXIT_FAILURE, 0, "first popped operand to '%s' must be a "
           "number (specifying how many datasets to use)", token);
 
-  /* Put the value into 'num'. */
+  /* Put the first-popped value into 'num'. */
   tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_SIZE_T);
   num=*(size_t *)(tmp->array);
   gal_data_free(tmp);
@@ -1079,43 +1072,88 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
       /* Things that differ from the first dataset and the rest. */
       if(out) /* Not the first. */
         {
-          /* Basic sanity checks. */
+          /* All entries have to have the same data type. */
           if(tmp->type!=out->type)
-            error(EXIT_FAILURE, 0, "the operands to '%s' have to have the "
-                  "same data type (the inputs contain atleast two types: "
-                  "'%s' and '%s')", token, gal_type_name(tmp->type, 1),
-                  gal_type_name(out->type, 1));
-          if( tmp->ndim!=out->ndim-1
-              || tmp->dsize[0]!=out->dsize[1]
-              || tmp->dsize[1]!=out->dsize[2] )
-            error(EXIT_FAILURE, 0, "the operands to '%s' have to have the "
-                  "same size", token);
+            error(EXIT_FAILURE, 0, "the operands to '%s' must have "
+                  "the same data type (the first popped operand has a "
+                  "'%s' type, but the popped operand number %zu has "
+                  "type '%s')", token, gal_type_name(out->type, 1), i+1,
+                  gal_type_name(tmp->type, 1));
+
+          /* All entries should also have the same dimension. */
+          if( gal_dimension_is_different(ref, tmp) )
+            error(EXIT_FAILURE, 0, "the operands to '%s' must have the "
+                  "same size in all dimensions", token);
         }
       else  /* First popped operand. */
         {
           /* First popped operand, do necessary basic checks here. */
-          if(tmp->ndim!=2)
-            error(EXIT_FAILURE, 0, "currently only 2-dimensional datasets "
-                  "are acceptable for '%s', please get in touch with us at "
-                  "%s so we add functionality for different dimensions",
-                  token, PACKAGE_BUGREPORT);
+          switch(operator)
+            {
+            case ARITHMETIC_OP_ADD_DIMENSION_SLOW:
+              dsize[0]=num;
+              dsize[1]=tmp->dsize[0];
+              dsize[2]=tmp->dsize[1];
+              nbytes=gal_type_sizeof(tmp->type)*tmp->size;
+              if(tmp->ndim!=2)
+                error(EXIT_FAILURE, 0, "currently only 2-dimensional "
+                      "datasets are acceptable for '%s', please get in "
+                      "touch with us at %s so we add functionality for "
+                      "different dimensions", token, PACKAGE_BUGREPORT);
+              break;
+            case ARITHMETIC_OP_ADD_DIMENSION_FAST:
+              dsize[1]=num;
+              dsize[0]=tmp->dsize[0];
+              nbytes=gal_type_sizeof(tmp->type);
+              if(tmp->ndim!=1)
+                error(EXIT_FAILURE, 0, "currently only 1-dimensional "
+                      "datasets are acceptable for '%s', please get in "
+                      "touch with us at %s so we add functionality for "
+                      "different dimensions", token, PACKAGE_BUGREPORT);
+              break;
+            default:
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                    "%s to address the problem. The operator code '%d' "
+                    "isn't recognized", __func__, PACKAGE_BUGREPORT,
+                    operator);
+            }
+
+          /* Allocate the size-reference dataset to simplify checking of
+             the sizes for each new dataset. This is done because this
+             dataset that is actually being popped will be freed right
+             after its values have been written into the output. We don't
+             care about any data from this dataset, and to avoid wasting
+             memory, we will simply allocate a one-element dataset. But
+             we'll many set its sizes to the size of the popped dataset. */
+          ref=gal_data_alloc(NULL, tmp->type, 1, &one, NULL, 0, -1, 1,
+                             NULL, NULL, NULL);
+          free(ref->dsize);
+          ref->ndim=tmp->ndim;
+          ref->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, ref->ndim, 0,
+                                          __func__, "ref->dsize");
+          for(j=0;j<ref->ndim;++j) ref->dsize[j]=tmp->dsize[j];
 
           /* Allocate the output dataset. */
-          dsize[0]=num;
-          dsize[1]=tmp->dsize[0];
-          dsize[2]=tmp->dsize[1];
-          out = gal_data_alloc(NULL, tmp->type, ndim, dsize, NULL, 0,
-                               p->cp.minmapsize, p->cp.quietmmap, NULL,
+          out = gal_data_alloc(NULL, tmp->type, tmp->ndim+1, dsize, NULL,
+                               0, p->cp.minmapsize, p->cp.quietmmap, NULL,
                                NULL, NULL);
-
-          /* Get the number of bytes in each dataset. */
-          nbytes=gal_type_sizeof(tmp->type)*tmp->size;
         }
 
       /* Copy the dataset into the higher-dimensional output. */
-      memcpy(gal_pointer_increment(out->array, (num-1-i)*tmp->size,
-                                   tmp->type),
-             tmp->array, nbytes);
+      switch(operator)
+        {
+        case ARITHMETIC_OP_ADD_DIMENSION_SLOW:
+          memcpy(gal_pointer_increment(out->array, (num-1-i)*tmp->size,
+                                       tmp->type),
+                 tmp->array, nbytes);
+          break;
+        case ARITHMETIC_OP_ADD_DIMENSION_FAST:
+          for(j=0;j<tmp->size;++j)
+            memcpy(gal_pointer_increment(out->array, j*num+num-i-1, out->type),
+                   gal_pointer_increment(tmp->array, j, out->type), nbytes);
+
+          break;
+        }
 
       /* Clean up. */
       gal_data_free(tmp);
@@ -1123,12 +1161,47 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
 
   /* Put the higher-dimensional output on the operands stack. */
   operands_add(p, NULL, out);
+
+  /* Clean up. */
+  gal_data_free(ref);
 }
 
 
 
 
 
+void
+arithmetic_repeat(struct arithmeticparams *p, char *token, int operator)
+{
+  size_t i, num;
+  gal_data_t *ttmp, *tmp = operands_pop(p, token);
+
+  /* Make sure the first operand is a number. */
+  if(tmp->size!=1)
+    error(EXIT_FAILURE, 0, "first popped operand to '%s' must be a "
+          "number (specifying how many datasets to use)", token);
+
+  /* Put the first-popped value into 'num'. */
+  tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_SIZE_T);
+  num=*(size_t *)(tmp->array);
+  gal_data_free(tmp);
+
+  /* Pop the dataset to be copied. */
+  tmp=operands_pop(p, token);
+
+  /* Add it to the stack for the desired number of times. */
+  for(i=0; i<num; ++i)
+    {
+      ttmp=gal_data_copy(tmp);
+      operands_add(p, NULL, ttmp);
+    }
+
+  /* Clean up. */
+  gal_data_free(tmp);
+}
+
+
+
 
 
 
@@ -1195,10 +1268,12 @@ arithmetic_set_operator(char *string, size_t *num_operands)
         { op=ARITHMETIC_OP_COLLAPSE_MEAN;         *num_operands=0; }
       else if (!strcmp(string, "collapse-number"))
         { op=ARITHMETIC_OP_COLLAPSE_NUMBER;       *num_operands=0; }
-      else if (!strcmp(string, "unique"))
-        { op=ARITHMETIC_OP_UNIQUE;                *num_operands=0; }
-      else if (!strcmp(string, "add-dimension"))
-        { op=ARITHMETIC_OP_ADD_DIMENSION;         *num_operands=0; }
+      else if (!strcmp(string, "add-dimension-slow"))
+        { op=ARITHMETIC_OP_ADD_DIMENSION_SLOW;    *num_operands=0; }
+      else if (!strcmp(string, "add-dimension-fast"))
+        { op=ARITHMETIC_OP_ADD_DIMENSION_FAST;    *num_operands=0; }
+      else if (!strcmp(string, "repeat"))
+        { op=ARITHMETIC_OP_REPEAT;                *num_operands=0; }
       else
         error(EXIT_FAILURE, 0, "the argument '%s' could not be "
               "interpretted as a file name, named dataset, number, "
@@ -1253,10 +1328,10 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
 
         case -1:
           /* This case is when the number of operands is itself an
-             operand. So except for sigma-clipping (that has other
-             parameters), the first popped operand must be an
-             integer number, we will use that to construct a linked
-             list of any number of operands within the single 'd1'
+             operand. So except for operators that have high-level
+             parameters (like sigma-clipping), the first popped operand
+             must be an integer number, we will use that to construct a
+             linked list of any number of operands within the single 'd1'
              pointer. */
           numop=pop_number_of_operands(p, operator, operator_string, &d2);
           for(i=0;i<numop;++i)
@@ -1264,10 +1339,10 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           break;
 
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-                "the problem. '%zu' is not recognized as an operand "
-                "counter (with '%s')", __func__, PACKAGE_BUGREPORT,
-                num_operands, operator_string);
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
+                "to fix the problem. '%zu' is not recognized as an "
+                "operand counter (with '%s')", __func__,
+                PACKAGE_BUGREPORT, num_operands, operator_string);
         }
 
       /* Run the arithmetic operation. Note that 'gal_arithmetic'
@@ -1328,12 +1403,13 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           arithmetic_collapse(p, operator_string, operator);
           break;
 
-        case ARITHMETIC_OP_UNIQUE:
-          arithmetic_unique(p, operator_string, operator);
+        case ARITHMETIC_OP_ADD_DIMENSION_SLOW:
+        case ARITHMETIC_OP_ADD_DIMENSION_FAST:
+          arithmetic_add_dimension(p, operator_string, operator);
           break;
 
-        case ARITHMETIC_OP_ADD_DIMENSION:
-          arithmetic_add_dimension(p, operator_string, operator);
+        case ARITHMETIC_OP_REPEAT:
+          arithmetic_repeat(p, operator_string, operator);
           break;
 
         default:
@@ -1379,11 +1455,12 @@ arithmetic_set_name_used_later(void *in, char *name)
 void
 reversepolish(struct arithmeticparams *p)
 {
-  gal_data_t *data;
   size_t num_operands=0;
   gal_list_str_t *token;
+  gal_data_t *data, *col;
   char *hdu, *filename, *printnum;
   int operator=GAL_ARITHMETIC_OP_INVALID;
+  struct gal_options_common_params *cp=&p->cp;
 
   /* Prepare the processing: */
   p->popcounter=0;
@@ -1412,6 +1489,11 @@ reversepolish(struct arithmeticparams *p)
       else if( !strncmp(token->v, GAL_ARITHMETIC_SET_PREFIX,
                         GAL_ARITHMETIC_SET_PREFIX_LENGTH) )
         gal_arithmetic_set_name(&p->setprm, token->v);
+      else if( (col=gal_arithmetic_load_col(token->v, cp->searchin,
+                                            cp->ignorecase,
+                                            cp->minmapsize,
+                                            cp->quietmmap))!=NULL )
+        operands_add(p, NULL, col);
       else if(    gal_array_file_recognized(token->v)
                || gal_arithmetic_set_is_name(p->setprm.named, token->v) )
         operands_add(p, token->v, NULL);
@@ -1424,6 +1506,7 @@ reversepolish(struct arithmeticparams *p)
           data->minmapsize=p->cp.minmapsize;
           operands_add(p, NULL, data);
         }
+
       /* Last option is an operator: the program will abort if the token
          isn't an operator. */
       else
@@ -1467,8 +1550,10 @@ reversepolish(struct arithmeticparams *p)
                                                   p->cp.minmapsize,
                                                   p->cp.quietmmap);
           data=p->operands->data;
-          data->ndim=gal_dimension_remove_extra(data->ndim, data->dsize, NULL);
-          if(!p->cp.quiet) printf(" - %s (hdu %s) is read.\n", filename, hdu);
+          data->ndim=gal_dimension_remove_extra(data->ndim, data->dsize,
+                                                NULL);
+          if(!p->cp.quiet)
+            printf(" - %s (hdu %s) is read.\n", filename, hdu);
         }
       else
         error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix "
@@ -1479,9 +1564,10 @@ reversepolish(struct arithmeticparams *p)
 
 
   /* If the final data structure has more than one element, write it as a
-     FITS file. Otherwise, print it in the standard output. */
+     FITS file. Otherwise, if the user didn't call '--output', print it in
+     the standard output. */
   data=p->operands->data;
-  if(data->size==1)
+  if(data->size==1 && data->ndim==1 && p->outnamerequested==0)
     {
       /* Make the string to print the number. */
       printnum=gal_type_to_string(data->array, data->type, 0);
@@ -1492,6 +1578,18 @@ reversepolish(struct arithmeticparams *p)
     }
   else
     {
+      /* If the user has requested a name, units or comments for the FITS
+         file, insert them into the dataset here. */
+      if(p->metaname)
+        { if(data->name) free(data->name);
+          gal_checkset_allocate_copy(p->metaname, &data->name); }
+      if(p->metaunit)
+        { if(data->unit) free(data->unit);
+          gal_checkset_allocate_copy(p->metaunit, &data->unit); }
+      if(p->metacomment)
+        { if(data->comment) free(data->comment);
+          gal_checkset_allocate_copy(p->metacomment, &data->comment); }
+
       /* Put a copy of the WCS structure from the reference image, it
          will be freed while freeing 'data'. */
       data->wcs=p->refdata.wcs;
@@ -1501,6 +1599,8 @@ reversepolish(struct arithmeticparams *p)
                         "ARITHMETIC", 0);
       else
         gal_fits_img_write(data, p->cp.output, NULL, PROGRAM_NAME);
+
+      /* Let the user know that the job is done. */
       if(!p->cp.quiet)
         printf(" - Write (final): %s\n", p->cp.output);
     }
