@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -190,13 +190,14 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
-      gal_list_str_add(&p->inputnames, arg, 0);
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
+      if(arg[0]!='\0') gal_list_str_add(&p->inputnames, arg, 0);
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -235,14 +236,17 @@ ui_colormap_sanity_check(struct converttparams *p)
   size_t nparams=0;
   int ccode=COLOR_INVALID;
 
-  /* See how many parameters are necessary. */
+  /* See how many parameters are necessary.
+     Notes for TAB completion:
+        1. Keep 'gray' and 'grey' in the same line.
+        2. Keep a space after the ',' before the strings.   */
   strarr=p->colormap->array;
-  if     ( !strcmp(strarr[0],"hsv"))     { ccode=COLOR_HSV;     nparams=2; }
-  else if( !strcmp(strarr[0],"sls"))     { ccode=COLOR_SLS;     nparams=0; }
-  else if( !strcmp(strarr[0],"viridis")) { ccode=COLOR_VIRIDIS; nparams=0; }
-  else if( !strcmp(strarr[0],"gray") || !strcmp(strarr[0],"grey"))
-                                      { ccode=COLOR_GRAY; nparams=0; }
-  else if( !strcmp(strarr[0],"sls-inverse"))
+  if     ( !strcmp(strarr[0], "hsv"))     { ccode=COLOR_HSV;     nparams=2; }
+  else if( !strcmp(strarr[0], "sls"))     { ccode=COLOR_SLS;     nparams=0; }
+  else if( !strcmp(strarr[0], "viridis")) { ccode=COLOR_VIRIDIS; nparams=0; }
+  else if( !strcmp(strarr[0], "gray") || !strcmp(strarr[0], "grey"))
+                                         { ccode=COLOR_GRAY; nparams=0; }
+  else if( !strcmp(strarr[0], "sls-inverse"))
     { ccode=COLOR_SLS_INVERSE; nparams=0; }
   else
     error(EXIT_FAILURE, 0, "'%s' not recognized as a colormap given "
@@ -337,7 +341,8 @@ ui_read_check_only_options(struct converttparams *p)
 
   if(p->fluxhighstr && p->fluxlowstr)
     {
-      cond=gal_arithmetic(GAL_ARITHMETIC_OP_GT, 1, GAL_ARITHMETIC_NUMOK,
+      cond=gal_arithmetic(GAL_ARITHMETIC_OP_GT, 1,
+                          GAL_ARITHMETIC_FLAG_NUMOK,
                           p->fluxhigh, p->fluxlow);
 
       if( *((unsigned char *)cond->array) == 0 )
@@ -515,7 +520,7 @@ ui_make_channels_ll(struct converttparams *p)
       if(strcmp(name->v, "blank")) gal_checkset_check_file(name->v);
 
       /* FITS: */
-      if( gal_fits_name_is_fits(name->v) )
+      if( gal_fits_file_recognized(name->v) )
         {
           /* Get the HDU value for this channel. */
           if(p->globalhdu)
@@ -733,11 +738,17 @@ ui_prepare_input_channels(struct converttparams *p)
   /* Make sure there are 1 (for grayscale), 3 (for RGB) or 4 (for CMYK)
      color channels. */
   if(p->numch!=1 && p->numch!=3 && p->numch!=4)
-    error(EXIT_FAILURE, 0, "the number of input color channels has to be "
-          "1 (for non image data, grayscale or only K channel in CMYK), "
-          "3 (for RGB) and 4 (for CMYK). You have given %zu color channels. "
-          "Note that some file formats (for example JPEG in RGB mode) can "
-          "contain more than one color channel", p->numch);
+    error(EXIT_FAILURE, 0, "the number of input color channels has to "
+          "be 1 (for non image data, grayscale or only K channel in "
+          "CMYK), 3 (for RGB) and 4 (for CMYK). You have given %zu "
+          "color channels. Note 1: some file formats (for example "
+          "JPEG in RGB mode) can contain more than one color channel, "
+          "if such a file is given all its channels are read, so "
+          "separate them first. Note 2: if your first input channel "
+          "was given through the standard input (piped from another "
+          "program) you can fix this error by giving a larger value "
+          "to the '--stdintimeout' option (currently %ld "
+          "micro-seconds)", p->numch, p->cp.stdintimeout);
 
 
   /* If there are multiple color channels, then ignore the monotocolor
