@@ -203,7 +203,8 @@ keywords_write_update(struct fitsparams *p, fitsfile **fptr,
         {
           if(u1w2==1)
             {
-              if(keyll->value)
+              if( keyll->value
+                  && gal_blank_is(keyll->value, keyll->type)==0 )
                 {
                   if( fits_update_key(*fptr,
                                       gal_fits_type_to_datatype(keyll->type),
@@ -220,7 +221,8 @@ keywords_write_update(struct fitsparams *p, fitsfile **fptr,
             }
           else if (u1w2==2)
             {
-              if(keyll->value)
+              if( keyll->value
+                  && gal_blank_is(keyll->value, keyll->type)==0 )
                 {
                   if( fits_write_key(*fptr,
                                      gal_fits_type_to_datatype(keyll->type),
@@ -372,14 +374,73 @@ keywords_verify(struct fitsparams *p, fitsfile **fptr)
 
 
 
-
+/* To copy keys to output file when '--copykeys' is given in 'STR,STR,STR'
+   format. */
 static void
-keywords_copykeys(struct fitsparams *p, char *inkeys, size_t numinkeys)
+keywords_copykeys_name(struct fitsparams *p, fitsfile *fptr,
+                       char *inkeys, size_t numinkeys,
+                       int *updatechecksum)
+{
+  size_t i, j;
+  int status=0, found;
+  char **strarr=p->copykeysname->array;
+
+  /* Copy the requested headers into the output files header. */
+  for(i=0; i<p->copykeysname->size; ++i)
+    {
+      /* Find the requested key from the set of input keywords that were
+         read before. The FITS standard does specify that the keyword
+         should only have upper-case letters, but CFITSIO does accept small
+         case (and write upper-case), so we'll also ignore case when
+         comparing.*/
+      found=0;
+      for(j=0;j<numinkeys-1;++j) /* numkeys-1: it includes 'END'. */
+        {
+          /* First check the first few characters, if they don't match, see
+             if it stars with 'HIERARCH' (which is placed before long
+             keyword names). */
+          if( !strncasecmp(strarr[i], &inkeys[j*80], strlen(strarr[i])) )
+            found=1;
+          else
+            {
+              if( !strncasecmp("HIERARCH", &inkeys[j*80], 8)
+                  && !strncasecmp(strarr[i], &inkeys[j*80+9],
+                                  strlen(strarr[i])) )
+                found=1;
+            }
+
+          /* Write the keyword if it was found. */
+          if(found==1)
+            {
+              if(fits_write_record(fptr, &inkeys[j*80], &status))
+                gal_fits_io_error(status, NULL);
+              else { *updatechecksum=1; break; }
+            }
+          }
+
+      /* If the key wasn't found after a full parsing of the keywords,
+         print an error message. */
+      if(found==0)
+        error(EXIT_FAILURE, 0, "%s: no keyword with name '%s' found",
+              gal_fits_name_save_as_string(p->input->v, p->cp.hdu),
+              strarr[i]);
+    }
+}
+
+
+
+
+
+/* To copy keys to output file when --copykeys are given in INT:INT
+    format. */
+static void
+keywords_copykeys_range(struct fitsparams *p, fitsfile *fptr,
+                        char *inkeys, size_t numinkeys,
+                        int *updatechecksum)
 {
   size_t i;
   long initial;
-  fitsfile *fptr;
-  int status=0, updatechecksum=0, checksumexists=0;
+  int status=0;
 
   /* Initial sanity check. Since 'numinkeys' includes 'END' (counting from
      1, as we do here), the first keyword must not be larger OR EQUAL to
@@ -414,6 +475,27 @@ keywords_copykeys(struct fitsparams *p, char *inkeys, size_t numinkeys)
           "this header (%zu, including the 'END' keyword)", p->input->v,
           p->cp.hdu, p->copykeysrange[1], numinkeys);
 
+
+  /* Copy the requested headers into the output. */
+  for(i=p->copykeysrange[0]-1; i<=p->copykeysrange[1]-1; ++i)
+    {
+      if( fits_write_record(fptr, &inkeys[i*80], &status ) )
+        gal_fits_io_error(status, NULL);
+      else *updatechecksum=1;
+    }
+}
+
+
+
+
+
+static void
+keywords_copykeys(struct fitsparams *p, char *inkeys, size_t numinkeys)
+{
+  int status=0;
+  fitsfile *fptr;
+  int updatechecksum=0, checksumexists=0;
+
   /* Open the output HDU. */
   fptr=gal_fits_hdu_open(p->cp.output, p->outhdu, READWRITE);
 
@@ -421,22 +503,20 @@ keywords_copykeys(struct fitsparams *p, char *inkeys, size_t numinkeys)
      wrote anything). */
   checksumexists=gal_fits_key_exists_fptr(fptr, "CHECKSUM");
 
-  /* Copy the requested headers into the output. */
-  for(i=p->copykeysrange[0]-1; i<=p->copykeysrange[1]-1; ++i)
-    {
-      if( fits_write_record(fptr, &inkeys[i*80], &status ) )
-        gal_fits_io_error(status, NULL);
-      else updatechecksum=1;
-    }
+  /* Call different functions depending on whether a list or range of
+      keywords is given. */
+  if(p->copykeysname)
+    keywords_copykeys_name(p, fptr, inkeys, numinkeys, &updatechecksum);
+  else
+    keywords_copykeys_range(p, fptr, inkeys, numinkeys, &updatechecksum);
 
   /* If a checksum existed, and we made changes in the file, we should
-     upate the checksum. */
+     update the checksum. */
   if(checksumexists && updatechecksum)
     if( fits_write_chksum(fptr, &status) )
       gal_fits_io_error(status, NULL);
 
   /* Close the output FITS file. */
-  status=0;
   if(fits_close_file(fptr, &status))
     gal_fits_io_error(status, NULL);
 }
@@ -1081,7 +1161,7 @@ keywords(struct fitsparams *p)
     }
 
 
-  /* If a range of keywords must be copied, get all the keywords as a
+  /* If a list/range of keywords must be copied, get all the keywords as a
      single string. */
   if(p->copykeys)
     {
