@@ -268,10 +268,28 @@ ui_check_copykeys(struct fitsparams *p)
           break;
           */
         default:
-          error(EXIT_FAILURE, 0, "value to '--copykeys' must only contain "
-                "integer numbers and these special characters between them: "
-                "':' when necessary. But it is '%s' (the first "
-                "non-acceptable character is '%c').\n", p->copykeys, *pt);
+
+          /* If a name is given, it will start with an alphabetic
+             character, a hyphen, or an underscore (we already know that
+             the key doesn't start with a digit, since it would break out
+             of the 'switch'). Recall that according to the FITS standard,
+             a keyword name can only contain digits, alphabetic characters,
+             '-', or '_'. See "Section 4.1.2.1. Keyword name" of the FITS
+             4.0 standard.*/
+          if( isalpha(*pt) || *pt=='-' || *pt=='_' ) break;
+
+          /* Print an error if control reaches here because  */
+          error(EXIT_FAILURE, 0, "value to '--copykeys' must either be a "
+                "range of numbers (for example '--copykeys=10:20') or a "
+                "set of names (for example '--copykeys=KEY1,KEY2', for "
+                "any number of names, note that the first keyword name "
+                "should not contain only digits, thus get interpreted "
+                "as a number). However, your given value ('%s') couldn't "
+                "be interpreted as any of the two (the first "
+                "non-acceptable character is '%c'). If this seems to be "
+                "a problem on the programming side, please report the "
+                "issue to us at '%s'", p->copykeys, *pt,
+                PACKAGE_BUGREPORT);
           break;
         }
 
@@ -282,8 +300,19 @@ ui_check_copykeys(struct fitsparams *p)
       printf("\n\n------\n%c: %ld (%s)\n", *pt, read, tailptr);
       */
 
-      /* Make sure if a number was read at all? */
-      if(tailptr==pt) continue;   /* No number was read!                 */
+      /* If the value couldn't be read as a number, and the first element
+         hasn't been read as a number, parse it as a set of comma-separated
+         strings and break out of the parsing overall. */
+      if(tailptr==pt)
+        {
+          if(p->copykeysrange[0]==GAL_BLANK_LONG)
+            {
+              p->copykeysname=gal_options_parse_csv_strings_raw(pt,
+                                                                NULL, 0);
+              break;
+            }
+          else continue;
+        }
 
       /* Put it in the correct place. */
       p->copykeysrange[ forl=='f' ? 0 : 1 ]=read;
@@ -291,23 +320,38 @@ ui_check_copykeys(struct fitsparams *p)
     }
 
   /* Basic sanity checks. */
-  if( p->copykeysrange[1]==GAL_BLANK_LONG )
-    error(EXIT_FAILURE, 0, "no ending keyword number given to '--copykeys'. "
-          "If you want to copy all the keywords after a certain one "
-          "(without worrying about how many there are), you can use '-1'.\n\n"
-          "For example if you want to copy all the keywords after the 20th, "
-          "you can use '--copykeys=20,-1'. Generally, you can use negative "
-          "numbers for the last keyword number to count from the end.");
-  if( p->copykeysrange[0]<=0 )
-    error(EXIT_FAILURE, 0, "the first number given to '--copykeys' must be "
-          "positive");
-  if( p->copykeysrange[1]>=0 && p->copykeysrange[0]>=p->copykeysrange[1] )
-    error(EXIT_FAILURE, 0, "the first number (%ld) given to '--copykeys' "
-          "must be smaller than the second (%ld)", p->copykeysrange[0],
-          p->copykeysrange[1]);
+  if( p->copykeysname==NULL )
+    {
+      if( p->copykeysrange[1]==GAL_BLANK_LONG )
+        error(EXIT_FAILURE, 0, "no ending keyword number given to "
+              "'--copykeys'. If you want to copy all the keywords after "
+              "a certain one (without worrying about how many there "
+              "are), you can use '-1'.\n\n"
+              "For example if you want to copy all the keywords after "
+              "the 20th, you can use '--copykeys=20,-1'. Generally, "
+              "you can use negative numbers for the last keyword "
+              "number to count from the end.");
+      if( p->copykeysrange[0]<=0 )
+        error(EXIT_FAILURE, 0, "the first number given to '--copykeys' "
+              "must be positive");
+      if( p->copykeysrange[1]>=0
+          && p->copykeysrange[0]>=p->copykeysrange[1] )
+        error(EXIT_FAILURE, 0, "the first number (%ld) given to "
+              "'--copykeys' must be smaller than the second (%ld)",
+              p->copykeysrange[0], p->copykeysrange[1]);
+    }
 
   /* For a check:
-  printf("copykeys: %ld, %ld\n", p->copykeysrange[0], p->copykeysrange[1]);
+  if( p->copykeysname )
+    {
+      size_t i;
+      char **strarr=p->copykeysname->array;
+      printf("copykeys: ");
+      for(i=0;i<p->copykeysname->size;++i)
+        printf("%s ", strarr[i]);
+    }
+  else
+    printf("copykeys: %ld, %ld\n", p->copykeysrange[0], p->copykeysrange[1]);
   exit(0);
   */
 }
