@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -41,6 +41,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Some constants */
 #define EPSREL_FOR_INTEG   2
 #define DEGREESTORADIANS   M_PI/180.0
+#define RADIANSTODEGREES   180.0/M_PI
+
 
 
 /* Modes to interpret coordinates. */
@@ -66,7 +68,9 @@ enum profile_types
   PROFILE_FLAT,                 /* Flat profile.               */
   PROFILE_CIRCUMFERENCE,        /* Circumference profile.      */
   PROFILE_DISTANCE,             /* Elliptical radius of pixel. */
-  PROFILE_CUSTOM,          /* Radial prof. in file/table. */
+  PROFILE_CUSTOM_PROF,          /* Radial prof. in file/table. */
+  PROFILE_AZIMUTH,              /* Azimuthal angle at distance.*/
+  PROFILE_CUSTOM_IMG,           /* Custom image to insert.     */
 
   PROFILE_MAXIMUM_CODE,         /* Just for a sanity check.    */
 };
@@ -115,8 +119,10 @@ struct mkprofparams
   char            *backname;  /* Name of background image file name.      */
   char             *catname;  /* Name of catalog of parameters.           */
   char             *backhdu;  /* HDU of background image.                 */
-  char          *customname;  /* Table to use for radial profile.         */
-  char           *customhdu;  /* HDU of table to use for radial profile.  */
+  char      *customprofname;  /* Table to use for radial profile.         */
+  char       *customprofhdu;  /* HDU of table to use for radial profile.  */
+  gal_list_str_t *customimgname;  /* Image to insert into the image.      */
+  gal_list_str_t  *customimghdu;  /* HDU of images for custom image.      */
   size_t             *dsize;  /* Size of the output image.                */
   uint8_t       clearcanvas;  /* Pixels in background image set to zero.  */
   gal_data_t        *kernel;  /* Parameters to define a kernel.           */
@@ -149,6 +155,8 @@ struct mkprofparams
   char                *tcol;  /* Truncation of the profiles.              */
   uint8_t       mforflatpix;  /* mcol is flat pixel value (f is 4 or 5).  */
   uint8_t  mcolisbrightness;  /* mcol is total brightness, not magnitude. */
+  uint8_t     mcolnocustimg;  /* mcol is ignored in 'custom-img'.         */
+  uint8_t    mcolnocustprof;  /* mcol is ignored in 'custom-profile'.     */
   gal_data_t         *crpix;  /* CRPIX FITS header keywords.              */
   gal_data_t         *crval;  /* CRVAL FITS header keywords.              */
   gal_data_t         *cdelt;  /* For CDELTi FITS header keywords.         */
@@ -188,7 +196,7 @@ struct mkprofparams
   pthread_cond_t     qready;  /* bq is ready to be written.               */
   pthread_mutex_t     qlock;  /* Mutex lock to change builtq.             */
   double          halfpixel;  /* Half pixel in oversampled image.         */
-  char           *wcsheader;  /* The WCS header information for main img. */
+  char              *wcsstr;  /* The WCS keywords derived from main img.  */
   int            wcsnkeyrec;  /* The number of keywords in the WCS header.*/
   char       *mergedimgname;  /* Name of merged image.                    */
   gal_data_t        *custom;  /* Table containing custom values.          */
