@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -75,6 +75,8 @@ enum option_keys_enum
   UI_KEY_PSFINIMG        = 1000,
   UI_KEY_MAGATPEAK,
   UI_KEY_MCOLISBRIGHTNESS,
+  UI_KEY_MCOLNOCUSTPROF,
+  UI_KEY_MCOLNOCUSTIMG,
   UI_KEY_MODE,
   UI_KEY_CCOL,
   UI_KEY_FCOL,
@@ -93,7 +95,9 @@ enum option_keys_enum
   UI_KEY_PC,
   UI_KEY_CUNIT,
   UI_KEY_CTYPE,
+  UI_KEY_CUSTOMIMG,
   UI_KEY_CUSTOMTABLE,
+  UI_KEY_CUSTOMIMGHDU,
   UI_KEY_CUSTOMTABLEHDU,
 };
 
