@@ -5,7 +5,7 @@ Warp is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -75,7 +75,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 
-/* Similar to 'nearestint_halflower' but:
+/* Similar to 'nearestint_halfhigher' but:
 
    nearestint_halflower(0.5f) --> 0.0f;
  */
@@ -183,7 +183,7 @@ warp_onthread(void *inparam)
       for(y=ystart;y<yend;++y)
         {
           /* If the pixel isn't in the image (note that the pixel
-             coordinates start from 1), contine to next. Note that the
+             coordinates start from 1), skip this pixel. Note that the
              pixel polygon should be counter clockwise. */
           if( y<1 || y>is0 ) continue;
           pcrn[1]=y-0.5f;      pcrn[3]=y-0.5f;
@@ -474,7 +474,8 @@ correct_wcs_save_output(struct warpparams *p)
     }
 
   /* Add the appropriate headers: */
-  gal_fits_key_write_filename("INF", p->inputname, &headers, 0);
+  gal_fits_key_write_filename("INF", p->inputname, &headers,
+                              0, p->cp.quiet);
   for(i=0;i<9;++i)
     {
       sprintf(&keyword[i*FLEN_KEYWORD], "WMTX%zu_%zu", i/3+1, i%3+1);
@@ -489,7 +490,8 @@ correct_wcs_save_output(struct warpparams *p)
   gal_fits_img_write(p->output, p->cp.output, headers, PROGRAM_NAME);
 
   /* Write the configuration keywords. */
-  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys,
+                              1, p->cp.quiet);
   gal_fits_key_write_config(&p->cp.okeys, "Warp configuration",
                             "WARP-CONFIG", p->cp.output, "0");
 }
