@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -46,6 +46,41 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /*********************************************************************/
 /*****************      Erosion and dilation      ********************/
 /*********************************************************************/
+static void
+binary_erode_dilate_1d(gal_data_t *input, int dilate0_erode1)
+{
+  size_t i, nr=input->dsize[0];
+  uint8_t f, b, *pt, *fpt, *byt=input->array;
+
+  /* Do a sanity check: */
+  if(dilate0_erode1!=1 && dilate0_erode1!=0)
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we can "
+          "fix this problem. The value to 'dilate0_erode1' is %u while it "
+          "should be 0 or 1", __func__, PACKAGE_BUGREPORT, dilate0_erode1);
+
+  /* Set the foreground and background values. */
+  if(dilate0_erode1==0) {f=1; b=0;}
+  else                  {f=0; b=1;}
+
+  /* Check the two extremes. */
+  if(byt[0]==b && byt[1]==f ) byt[0]=GAL_BINARY_TMP_VALUE;
+  if(nr>2)
+    if(byt[nr-1]==b && byt[nr-2]==f) byt[nr-1] = GAL_BINARY_TMP_VALUE;
+
+  /* Go over the full array. */
+  for(i=1;i<nr;++i)
+    if( byt[i]==b && ( byt[i-1]==f || byt[i+1]==f ) )
+      byt[i]=GAL_BINARY_TMP_VALUE;
+
+  /* Set all the changed pixels to the proper values: */
+  fpt=(pt=byt)+nr;
+  do *pt = *pt==GAL_BINARY_TMP_VALUE ? f : *pt; while(++pt<fpt);
+}
+
+
+
+
+
 static void
 binary_erode_dilate_2d_4con(gal_data_t *input, int dilate0_erode1)
 {
@@ -290,6 +325,10 @@ binary_erode_dilate(gal_data_t *input, size_t num, int connectivity,
   /* Go over every element and do the erosion. */
   switch(binary->ndim)
     {
+    case 1:
+      binary_erode_dilate_1d(binary, d0e1);
+      break;
+
     case 2:
       for(counter=0;counter<num;++counter)
         switch(connectivity)
@@ -653,7 +692,6 @@ gal_binary_connected_adjacency_matrix(gal_data_t *adjacency,
         ++curlab;
       }
 
-
   /* For a check.
   printf("=== Old labels --> new labels ===\n");
   for(i=1;i<num;++i) printf("%zu: %u\n", i, newlabs[i]);
@@ -668,6 +706,84 @@ gal_binary_connected_adjacency_matrix(gal_data_t *adjacency,
 
 
 
+/* List-based adjacency matrix: when the number of points to find adjacent
+   elements is large, the array-based adjacency solution of
+   'gal_binary_connected_adjacency_matrix' will consume far too much memory
+   (since it is a square). In those cases, we need to work based on
+   lists.
+
+   The input is an array of 'size_t' list pointers. Each one points to the
+   labels that are touching it.
+
+                     indexs        b       d
+                                   ^       ^
+                     indexs        a   b   c   a
+                                   ^   ^   ^   ^
+                     listarr:    | * | * | * | * |
+*/
+gal_data_t *
+gal_binary_connected_adjacency_list(gal_list_sizet_t **listarr,
+                                    size_t number, size_t minmapsize,
+                                    int quietmmap, size_t *numconnected)
+{
+  size_t i, p;
+  gal_list_sizet_t *tmp;
+  gal_data_t *newlabs_d;
+  gal_list_sizet_t *Q=NULL;
+  int32_t *newlabs, curlab=1;
+
+  /* Allocate (and clear) the output datastructure. */
+  newlabs_d=gal_data_alloc(NULL, GAL_TYPE_INT32, 1, &number, NULL, 1,
+                           minmapsize, quietmmap, NULL, NULL, NULL);
+  newlabs=newlabs_d->array;
+
+  /* Go over the input matrix and apply the same principle as we used to
+     identify connected components in an image: through a queue, find those
+     elements that are connected. */
+  for(i=1;i<number;++i)
+    if(newlabs[i]==0)
+      {
+        /* Add this old label to the list that must be corrected. */
+        gal_list_sizet_add(&Q, i);
+
+        /* Continue while the list has elements. */
+        while(Q!=NULL)
+          {
+            /* Pop the top old-label from the list. */
+            p=gal_list_sizet_pop(&Q);
+
+            /* If it has already been labeled then ignore it. */
+            if( newlabs[p]!=curlab )
+              {
+                /* Give it the new label. */
+                newlabs[p]=curlab;
+
+                /* Go over the adjacency list for this touching object and
+                   see if there are any not-yet-labeled objects that are
+                   touching it. */
+                for(tmp=listarr[p]; tmp!=NULL; tmp=tmp->next)
+                  if( newlabs[tmp->v]==0 )
+                    gal_list_sizet_add(&Q, tmp->v);
+              }
+          }
+
+        /* Increment the current label. */
+        ++curlab;
+      }
+
+  /* For a check.
+  printf("=== Old labels --> new labels ===\n");
+  for(i=1;i<number;++i) printf("%zu: %u\n", i, newlabs[i]);
+  */
+
+  /* Return the output. */
+  *numconnected = curlab-1;
+  return newlabs_d;
+}
+
+
+
+
 
 
 
